#!/usr/bin/perl -w

  ##
  ## Network Traffic Analyzer
  ## ~~~~~~~~~~~~~~~~~~~~~~~~
  ## http://nta.kyberdigi.cz/
  ## (C) C. McCohy <mccohy@kyberdigi.cz>
  ##
  ## This is the one-way script of Network Traffic Analyzer
  ## Please do not edit this file if you do NOT know exactly what you are doing
  ##
  ## This script should be ran every 5 minutes, which can be done via
  ##   the 'crontab -e' command. In the crontab editor put this line
  ##   */5 * * * * (cd /full/path/to/this/directory; ./nta.pl)
  ##
  ## Do not forget to edit the config.pl file before first running
  ##
  ## You don't need to be root to examine this script
  ##
  ## Good luck ;-)
  ##


  ##########################
  ## Basic initialization ##
  ##########################


  # Includes, variables declarations

  use strict;
  use vars qw (%netdevices %config);
  use POSIX qw (mktime strftime);
  use NTAFunc;
  use NTAGraph;
  use NTALock;
  do('config.pl');

  my (%dates, %templates, %hostinfo, %newdata);
  my (%data, %xdata);
  my @acttime=localtime();


  ## Lockfile test

#  &NTALock::exists($config{'file_lock'}) && &NTAFunc::out("Program zablokovan LOCKem. Odstrante ".$config{'file_lock'}." aby se program mohl spustit.");
#  &NTALock::lock($config{'file_lock'}) || &NTAFunc::out("Nemuzu vytvorit LOCK soubor ".$config{'file_lock'});

  ## Some configuration variables tests

  my $cnt=%netdevices;
  $cnt=~/^0/ && &NTAFunc::out('Promenna %netdevices v konfiguracnim souboru je prazdna');

  foreach my $file (grep { /^bin_/ } keys %config) {
    ((-f $config{$file}) && (-x $config{$file})) || 
      &NTAFunc::out("Soubor ".$config{$file}." nemuze byt nalezen nebo spusten");
  }

  foreach my $file (grep { /^(proc|template)_/ } keys %config) {
    (-r $config{$file}) || 
      &NTAFunc::out("Soubor ".$config{$file}." nelze precist");
  }

  (-d $config{'output_dir'}) || &NTAFunc::out("Vystupni adresar ".$config{'output_dir'}." neni adresar");

  foreach my $color (grep { /^graph_color_/ } keys %config) {
    $config{$color}=~/^#[a-f0-9]{6}$/i ||
      &NTAFunc::out("Konfiguracni promenna $color musi byt v RGB formatu (#rrggbb)");
  }

  (int($config{'show_back_main'}) && ($config{'show_back_main'}>0)) || do {
    &NTAFunc::warning("Konfiguracni promenna show_back_main musi byt platne cislo vetsi nez 0\nPouziji hodnotu 12, ale opravte konfiguraci v souboru config.pl prosim.");
    $config{'show_back_main'}=12;
  }; 

  ## Get dates

  $dates{'date'}=strftime($config{'date_date'}, localtime());
  $dates{'datetime'}=strftime($config{'date_datetime'}, localtime());
  $dates{'sysdate'}=strftime("%Y-%m-%d", localtime());
  $dates{'sysdatetime'}=strftime("%Y-%m-%d %H:%M:%S", localtime());
  $dates{'systime'}=strftime("%H:%M:%S", localtime());

  ## Get templates

  undef $/;
  foreach my $file ('template_header', 'template_footer', 'template_table') {
    open(TEMPLATE, $config{$file}) || &NTAFunc::out("Nemohu otevrit docasny soubor ".$config{$file});
    my $templatekey=$file; $templatekey=~s/^[^_]+_//s;
    $templates{$templatekey}=<TEMPLATE>;
    close(TEMPLATE);
    foreach my $key (grep { /^prg_/ } keys %config) {
      my $value=$config{$key};
      $templates{$templatekey}=~s/{$key}/$value/sgi;
    }
    foreach my $key (keys %dates) {
      my $value=$dates{$key};
      $templates{$templatekey}=~s/{$key}/$value/sgi;
    }
  }
  $/="\n";

  my $graphhrefs;
  foreach my $type ('hour', 'day', 'week', 'month') {
    $config{'show_back_'.$type} || next;
    $graphhrefs.="<a href=\"".$type.$config{'output_ext'}."\"><img border=\"0\" alt=\"Poslednich ".int($config{'show_back_'.$type}).' '.$type.($config{'show_back_'.$type}>1 ? '':'')." - kompletni graficka statistika\" src=\"graph-$type.png\"></a>\n";
  }
  $templates{'header'}=~s/{graphhrefs}/$graphhrefs/;

  ## Get host informations

  if($config{'output_hostinfo'}) {
    foreach my $key ('uname', 'uptime', 'hostname') { $hostinfo{$key}=''; }

    if(open(UNAME, $config{'proc_uname'})) {
      $_=<UNAME>;
      /^([^ ]+) [^ ]+ ([^ ]+) [^#]+(#[0-9]+) / && ($hostinfo{'uname'}="$1 $2 $3");
      close(UNAME);
    } else {
      &NTAFunc::warning("Nemohu cist z ".$config{'proc_uname'});
    }

    if(open(UPTIME, $config{'proc_uptime'})) {
      $_=<UPTIME>;
      /^([0-9]+)\./ && do {
        my $days=int($1/(3600*24));
        my $hours=int(($1-($days*24*3600))/3600);
        my $mins=int(($1-($hours*3600)-($days*24*3600))/60);
#        $hostinfo{'uptime'}="$days dni".($days==1 ? '':'s').sprintf(", %02d:%02d", $hours, $mins);
        $hostinfo{'uptime'}="$days dni".sprintf(", %02d:%02d", $hours, $mins);
      };
      close(UPTIME);
    } else {
      &NTAFunc::warning("Nemohu precist ".$config{'proc_uptime'});
    }

    $hostinfo{'hostname'}=`$config{'bin_hostname'}`; 
    $hostinfo{'hostname'} && chomp($hostinfo{'hostname'});
  }


  ## Clean new data

  foreach my $device (keys %netdevices) {
    $newdata{$device}{'ip'}='';
    $newdata{$device}{'mac'}='';
    $newdata{$device}{'in'}=0;
    $newdata{$device}{'out'}=0;
  }


  ###########################
  ## Traffic data handling ##
  ###########################


  ## Get new data about network traffic

  if($config{'output_ip'} || $config{'output_mac'}) {
    foreach my $device (keys %netdevices) {
      open(IFCONFIG, $config{'bin_ifconfig'}." $device |") || &NTAFunc::out("Nemohu spustit ".$config{'bin_ifconfig'});
      $_=<IFCONFIG>;
      chomp;
      $config{'output_mac'} && / ([0-9a-f:]+)\W*$/i && ($newdata{$device}{'mac'}=$1);
      $_=<IFCONFIG>;
      $config{'output_ip'} && /^[^:]+:([0-9.]{7,15}) / && ($newdata{$device}{'ip'}=$1);
      close(IFCONFIG);
    }
  }

  open(DEV, $config{'proc_netdev'}) || &NTAFunc::out("Nemohu precist ".$config{'proc_netdev'});
  <DEV>;
  <DEV>;
  while(<DEV>) {
    chomp; 
    s/ +/ /s;
    s/^ +//s;
    /^([^:]+):/ || next;
    my $device=$1;
    s/^[^:]+: *(.+)$/$1/;
    my @vals=split(' ');
    $newdata{$device}{'in'}=$vals[0];
    $newdata{$device}{'out'}=$vals[8];
  }
  close(DEV);


  ## Get data from main datafile (daily text stats)

  if(open(DATA, $config{'file_main'})) {
    while(<DATA>) {
      chomp;
      /^([^ ]+) ([0-9]{4}-[0-9]{2}-[0-9]{2}) (in|out) ([0-9]+)$/ && do {
        $data{'main'}{$1}{$2}{$3}=int($4);
        next;
      };
      /^([^ ]+) last (in|out) ([0-9]+)$/ && do { 
        $data{'main'}{$1}{'last'}{$2}=int($3);
        next;
      };
      /^(^[^ ]+) start (.+)$/ && do {
        $newdata{$1}{'start'}=$2;
        next;
      };
      /^(^[^ ]+) sysstart (.+)$/ && do {
        $newdata{$1}{'sysstart'}=$2;
        next;
      };
    }
    close(DATA);
  }


  ## Look if we read device for the first time

  foreach my $device (keys %netdevices) {
    foreach my $direction ('in', 'out') {
      $data{'main'}{$device}{'last'}{$direction} || ($data{'main'}{$device}{'last'}{$direction}=$newdata{$device}{$direction});
      if(!$newdata{$device}{'start'}) {
        $newdata{$device}{'start'}=$dates{'datetime'};
        $newdata{$device}{'sysstart'}=$dates{'sysdate'};
      }
    }
  }

  ## Fill empty slots

  foreach my $device (keys %netdevices) {
    my $lastdate;
    foreach my $date (sort keys %{$data{'main'}{$device}}) {
      $date=~/^([0-9]{4})-([0-9]{2})-([0-9]{2})$/ || next;
      my ($y, $m, $d)=($1, $2, $3);
      !$lastdate && do { $lastdate=$date; next; };
      my $testdate=strftime("%Y-%m-%d", localtime(mktime(0, 0, 0, $d-1, $m-1, $y-1900)));
      while($testdate ne $lastdate) {
        defined($data{'main'}{$device}{$testdate}{'in'}) || ($data{'main'}{$device}{$testdate}{'in'}=0);
        defined($data{'main'}{$device}{$testdate}{'out'}) || ($data{'main'}{$device}{$testdate}{'out'}=0);
        $d--;
        $testdate=strftime("%Y-%m-%d", localtime(mktime(0, 0, 0, $d-1, $m-1, $y-1900)));
      }
      $lastdate=$date;
    }
  }

  ## Count new traffic

  foreach my $device (keys %netdevices) {
    foreach my $direction ('in', 'out') {
      my $newvalue=$newdata{$device}{$direction};
      $data{'main'}{$device}{'last'}{$direction} &&
        ($data{'main'}{$device}{'last'}{$direction}<=$newdata{$device}{$direction}) &&
          ($newvalue=$newdata{$device}{$direction}-$data{'main'}{$device}{'last'}{$direction});
      $data{'main'}{$device}{$dates{'sysdate'}}{$direction}+=$newvalue;
    }
  }


  ## Count monthly summages

  foreach my $device (keys %netdevices) {
    foreach my $date (keys %{$data{'main'}{$device}}) {
      $date=~/^([0-9]{4})-([0-9]{2})-[0-9]{2}$/ || next;
      my ($y, $m)=($1, $2);
      foreach my $direction ('in', 'out') {
        $data{'text'}{$device}{"$y-$m"}{$direction}+=$data{'main'}{$device}{$date}{$direction};
      }
    }
  }


  ## Save new data

  my $start=strftime("%Y-%m-%d", localtime(mktime($acttime[0], $acttime[1], $acttime[2], $acttime[3], $acttime[4]-$config{'show_back_main'}, $acttime[5])));

  open(DATA, ">".$config{'file_main'}) || &NTAFunc::out("Nemohu zapsat do ".$config{'file_main'});
  foreach my $device (keys %netdevices) {
    foreach my $date (keys %{$data{'main'}{$device}}) {
      if($date=~/^[0-9]{4}-[0-9]{2}-[0-9]{2}$/ && ($date ge $start)) {
        foreach my $direction ('in', 'out') {
          print DATA "$device $date $direction ".$data{'main'}{$device}{$date}{$direction}."\n";
        }
      }
    }
    foreach my $direction ('in', 'out') {
      print DATA "$device last $direction ".$newdata{$device}{$direction}."\n";
    }
    print DATA "$device start ".$newdata{$device}{'start'}."\n";
    print DATA "$device sysstart ".$newdata{$device}{'sysstart'}."\n";
  }
  close(DATA);


  ## Get data for hourly, daily, weekly and monthly graph

  foreach my $type ('hour', 'day', 'week', 'month') {
    if(!$config{'show_back_'.$type}) {
      unlink $config{'file_'.$type};
    } else {

      ## Set startdate (only dates greather than this date will be read)

      my $startdate;
      foreach($type) {
        /^hour$/ && do {
          $startdate=strftime("%Y-%m-%d %H", localtime(mktime($acttime[0], $acttime[1], $acttime[2]-$config{'show_back_hour'}, $acttime[3], $acttime[4], $acttime[5])));
          last;
        };
        /^day$/ && do {
          $startdate=strftime("%Y-%m-%d %H", localtime(mktime($acttime[0], $acttime[1], $acttime[2], $acttime[3]-$config{'show_back_day'}, $acttime[4], $acttime[5])));
          last;
        };
        /^week$/ && do {
          $startdate=strftime("%Y-%m-%d", localtime(mktime($acttime[0], $acttime[1], $acttime[2], ($acttime[3]-$config{'show_back_week'}*7), $acttime[4], $acttime[5])));
          last;
        };
        /^month$/ && do {
          $startdate=strftime("%Y-%m-%d", localtime(mktime($acttime[0], $acttime[1], $acttime[2], $acttime[3], $acttime[4]-$config{'show_back_month'}, $acttime[5])));
          last;
        };
      }

      # Fill all data with zeroes (for full graph creation)

      foreach my $device (keys %netdevices) {
        foreach my $direction ('in', 'out') {
          $data{$type}{$device}{'last'}{$direction}=0;
        }
      }

      if($dates{'sysdatetime'}=~/^([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2}):([0-9]{2}):[0-9]{2}$/) {
        my ($y, $m, $d, $hour, $min)=($1, $2, $3, $4, $5);
        foreach ($type) {
          /^hour$/ && do {
            my $tail=sprintf("%02d", int($min/5));
            my $actdate=$dates{'sysdate'}." $hour:$tail";
            while($actdate ge $startdate.':00') {
              foreach my $device (keys %netdevices) {
                foreach my $direction ('in', 'out') {
                  $data{$type}{$device}{$actdate}{$direction}=0;
                }
              }
              if(int($tail)) { $tail--; } else { $tail=11; $hour--; }
              $actdate=strftime("%Y-%m-%d %H:", localtime(mktime(0, 0, $hour, $d, $m-1, $y-1900))).sprintf("%02d", $tail);
            }
            last;
          };
          /^day$/ && do {
            my $tail=int($min/30);
            my $actdate=$dates{'sysdate'}." $hour:$tail";
            while($actdate ge $startdate.':0') {
              foreach my $device (keys %netdevices) {
                foreach my $direction ('in', 'out') {
                  $data{$type}{$device}{$actdate}{$direction}=0;
                }
              }
              if(int($tail)) { $tail--; } else { $tail=1; $hour--; }
              $actdate=strftime("%Y-%m-%d %H:", localtime(mktime(0, 0, $hour, $d, $m-1, $y-1900))).$tail;
            }
            last;
          };
          /^week$/ && do {
            my $tail=sprintf("%02d", int($hour/2));
            my $actdate=$dates{'sysdate'}." $tail";
            while($actdate ge $startdate.' 00') {
              foreach my $device (keys %netdevices) {
                foreach my $direction ('in', 'out') {
                  $data{$type}{$device}{$actdate}{$direction}=0;
                }
              }
              if(int($tail)) { $tail--; } else { $tail=11; $d--; }
              $actdate=strftime("%Y-%m-%d ", localtime(mktime(0, 0, 0, $d, $m-1, $y-1900))).sprintf("%02d", $tail);
            }
            last;
          };
          /^month$/ && do {
            my $tail=int($hour/12);
            my $actdate=$dates{'sysdate'}." $tail";
            while($actdate ge $startdate.' 0') {
              foreach my $device (keys %netdevices) {
                foreach my $direction ('in', 'out') {
                  $data{$type}{$device}{$actdate}{$direction}=0;
                }
              }
              if(int($tail)) { $tail--; } else { $tail=1; $d--; }
              $actdate=strftime("%Y-%m-%d ", localtime(mktime(0, 0, 0, $d, $m-1, $y-1900))).$tail;
            }
            last;
          };
        }
      }


      # Get data from graphdatafile

      if(open(GRAPHDATA, $config{'file_'.$type})) {
        while(my $line=<GRAPHDATA>) {
          chomp($line);
          ($type eq 'hour') && ($line=~/^([0-9]{4}-[0-9]{2}-[0-9]{2}) ([0-9]{2}):([0-9]{2}) ([^ ]+) (in|out) ([0-9]+)$/) && ("$1 $2" ge $startdate) && do {
            $data{$type}{$4}{"$1 $2:$3"}{$5}=$6;
            next;
          };
          ($type eq 'day') && ($line=~/^([0-9]{4}-[0-9]{2}-[0-9]{2}) ([0-9]{2}):([01]) ([^ ]+) (in|out) ([0-9]+)$/) && ("$1 $2" ge $startdate) && do {
            $data{$type}{$4}{"$1 $2:$3"}{$5}=$6;
            next;
          };
          ($type eq 'week') && ($line=~/^([0-9]{4}-[0-9]{2}-[0-9]{2}) ([0-9]{2}) ([^ ]+) (in|out) ([0-9]+)$/) && ("$1" ge $startdate) && do {
            $data{$type}{$3}{"$1 $2"}{$4}=$5;
            next;
          };
          ($type eq 'month') && ($line=~/^([0-9]{4}-[0-9]{2}-[0-9]{2}) ([01]) ([^ ]+) (in|out) ([0-9]+)$/) && ("$1 $2" ge $startdate) && do {
            $data{$type}{$3}{"$1 $2"}{$4}=$5;
            next;
          };
          $line=~/^([^ ]+) last (in|out) ([0-9]+)$/ && do {
            $data{$type}{$1}{'last'}{$2}=$3;
            next;
          };
        }
        close(GRAPHDATA);
      }

      # Test if we see device data for the first time

      foreach my $device (keys %netdevices) {
        foreach my $direction ('in', 'out') {
          $data{$type}{$device}{'last'}{$direction} || ($data{$type}{$device}{'last'}{$direction}=$newdata{$device}{$direction});
        }
      }

      # Count new traffic

      foreach my $device (keys %netdevices) {
        foreach my $direction ('in', 'out') {
          my $newvalue=$newdata{$device}{$direction};
          $data{$type}{$device}{'last'}{$direction} &&
            ($data{$type}{$device}{'last'}{$direction}<=$newvalue) &&
              ($newvalue-=$data{$type}{$device}{'last'}{$direction});

          $dates{'systime'}=~/^([0-9]{2}):([0-9]{2}):[0-9]{2}$/;
          my ($hour, $min)=($1, $2);
          foreach($type) {
            /^hour$/ && ($data{$type}{$device}{$dates{'sysdate'}." $hour:".sprintf("%02d", int($min/5))}{$direction}+=$newvalue);
            /^day$/ && ($data{$type}{$device}{$dates{'sysdate'}." $hour:".int($min/30)}{$direction}+=$newvalue);
            /^week$/ && ($data{$type}{$device}{$dates{'sysdate'}.' '.sprintf("%02d", int($hour/2))}{$direction}+=$newvalue);
            /^month$/ && ($data{$type}{$device}{$dates{'sysdate'}.' '.int($hour/12)}{$direction}+=$newvalue);
          }
        }
      }

      # Save data in graphdatafile again

      open(DATA, ">".$config{'file_'.$type}) || &NTAFunc::out("Cannot write to ".$config{'file_'.$type});
      foreach my $device (keys %netdevices) {
        foreach my $date (keys %{$data{$type}{$device}}) {
          if($date ne 'last') {
            foreach my $direction ('in', 'out') {
              print DATA "$date $device $direction ".$data{$type}{$device}{$date}{$direction}."\n";
            }
          }
        }
        foreach my $direction ('in', 'out') {
          print DATA "$device last $direction ".$newdata{$device}{$direction}."\n";
        }
      }
      close(DATA);

    }
  }



  #################
  ## HTML output ##
  #################
  
  ## Creating index page

  my $tabledata;

  open(INDEX, ">".$config{'output_dir'}.'/'.$config{'output_index'}.$config{'output_ext'}) || &NTAFunc::out("Nemohu zapsat do ".$config{'output_dir'}.'/'.$config{'output_index'}.$config{'output_ext'});
  print INDEX $templates{'header'};

  if($config{'output_hostinfo'} && ($hostinfo{'uname'} || $hostinfo{'uptime'} || $hostinfo{'hostname'})) {
    $tabledata=
      "<table>\n".
      ($hostinfo{'hostname'} ?
        "  <tr>\n".
        "    <td align=\"right\"><b>Jmeno pocitace:</b></td>\n".
        "    <td>".$hostinfo{'hostname'}."</td>\n".
        "  </tr>\n"
      :'').
      ($hostinfo{'uptime'} ?
        "  <tr>\n".
        "    <td align=\"right\"><b>Uptime pocitace:</b></td>\n".
        "    <td>".$hostinfo{'uptime'}."</td>\n".
        "  </tr>\n"
      :'').
      ($hostinfo{'uname'} ?
        "  <tr>\n".
        "    <td align=\"right\"><b>Verze kernelu:</b></td>\n".
        "    <td>".$hostinfo{'uname'}."</td>\n".
        "  </tr>\n"
      :'').
      "</table>\n";
    print INDEX &NTAFunc::table($templates{'table'}, "Informace o pocitaci", $tabledata)."<hr noshade>\n\n";
  }

  foreach my $device (keys %netdevices) {
    $tabledata=
      "<table border=\"0\">".
      ($newdata{$device}{'ip'} ? "<tr><td>IP adresa: </td><td><b>".$newdata{$device}{'ip'}."</b></td></tr>\n":'').
      ($newdata{$device}{'mac'} ? "<tr><td>MAC adresa: </td><td><b>".$newdata{$device}{'mac'}."</b></td></tr>\n":'').
      "<tr><td>Statistika bezi od: </td><td>".$newdata{$device}{'start'}."</tr>\n".
      "</table><dl><dd>\n".
      "<table ".$config{'output_ttag_table'}.">\n".
      "  <tr ".$config{'output_ttag_throw'}.">\n".
      "    <th>Mesic</th>\n".
      "    <th>Bytu&nbsp;VSTUP</th>\n".
      "    <th>Bytu&nbsp;VYSTUP</th>\n".
      "  </tr>\n";

    my($total_in, $total_out)=(0, 0);
    foreach my $date (sort {$b cmp $a} keys %{$data{'text'}{$device}}) {
      $date=~/^([0-9]{4})-([0-9]{2})$/ || next;
      my ($y, $m)=($1, $2);
      $tabledata.=
        "  <tr>\n".
        "    <td><a href=\"$device-$y-$m".$config{'output_ext'}."\">".strftime("%b %Y", localtime(mktime(0, 0, 0, 1, ($m-1), ($y-1900))))."</a></td>\n".
        "    <td align=\"right\">".&NTAFunc::nicenum($data{'text'}{$device}{$date}{'in'})."&nbsp;B</td>\n".
        "    <td align=\"right\">".&NTAFunc::nicenum($data{'text'}{$device}{$date}{'out'})."&nbsp;B</td>\n".
        "  </tr>\n";
      $total_in+=$data{'text'}{$device}{$date}{'in'};
      $total_out+=$data{'text'}{$device}{$date}{'out'};
    }

    my $hrefs;
    foreach my $type ('hour', 'day', 'week', 'month') {
      $config{'show_back_'.$type} || next;
      $hrefs.="<a href=\"$device-".$type.$config{'output_ext'}."\"><img border=\"0\" alt=\"Poslednich ".int($config{'show_back_'.$type}).' '.$type.($config{'show_back_'.$type}>1 ? '':'')." - graficka statistika\" src=\"graph-$type.png\"></a>\n";
    }
    $hrefs && ($hrefs="<a href=\"$device-graph".$config{'output_ext'}."\"><img border=\"0\" alt=\"Kompletni graficka statistika rozhrani\" src=\"graph.png\"></a>\n".$hrefs);

    $tabledata.=
      "  <tr valign=\"top\" align=\"right\" ".$config{'output_ttag_throw'}.">\n".
      "    <td><b>Celkem</b></td>\n".
      "    <td>".&NTAFunc::nicenum($total_in)."&nbsp;B<br><b>".&NTAFunc::b2xb($total_in)."B</b></td>\n".
      "    <td>".&NTAFunc::nicenum($total_out)."&nbsp;B<br><b>".&NTAFunc::b2xb($total_out)."B</b></td>\n".
      "  </tr>\n".
      "</table>\n".
      "</dd></dl>\n".
      "<p>\n".
      "$hrefs\n".
      "<p>\n";
    print INDEX &NTAFunc::table($templates{'table'}, $netdevices{$device}." ($device)", $tabledata)."<p>\n\n";
  }
  print INDEX $templates{'footer'};
  close(INDEX);


  ## Creating text monthly stats

  foreach my $device (keys %netdevices) {
    foreach my $date (keys %{$data{'text'}{$device}}) {
      open(MONTH, ">".$config{'output_dir'}."/$device-$date".$config{'output_ext'}) || &NTAFunc::out("Nemohu zapsat do ".$config{'output_dir'}."/$date".$config{'output_ext'});
      $tabledata=
        "<table border=\"0\">".
        ($newdata{$device}{'ip'} ? "<tr><td>IP adresa: </td><td><b>".$newdata{$device}{'ip'}."</b></td></tr>\n":'').
        ($newdata{$device}{'mac'} ? "<tr><td>MAC adresa: </td><td><b>".$newdata{$device}{'mac'}."</b></td></tr>\n":'').
        "<tr><td>Statistika bezi od: </td><td>".$newdata{$device}{'start'}."</tr>\n".	
        "</table>".	
        "<dl><dd>\n".
        "<table ".$config{'output_ttag_table'}.">\n".
        "  <tr ".$config{'output_ttag_throw'}.">\n".
        "    <th>Den</th>\n".
        "    <th>Bytu&nbsp;VSTUP</th>\n".
        "    <th>Bytu&nbsp;VYSTUP</th>\n".
        "  </tr>\n";
      
      $date=~/^([0-9]{4})-([0-9]{2})$/;
      foreach my $day (1..&NTAFunc::daysinmonth($1, $2)) {
        if(($date.sprintf("-%02d", $day) le $dates{'sysdate'}) && ($date.sprintf("-%02d", $day) ge $newdata{$device}{'sysstart'})) {
          $tabledata.=
            "  <tr align=\"right\">\n".
            "    <td>$day.</a></td>\n".
            "    <td>".&NTAFunc::nicenum($data{'main'}{$device}{$date.sprintf("-%02d", $day)}{'in'})."&nbsp;B</td>\n".
            "    <td>".&NTAFunc::nicenum($data{'main'}{$device}{$date.sprintf("-%02d", $day)}{'out'})."&nbsp;B</td>\n".
            "  </tr>\n";
        }
      }

      my $hrefs;
      foreach my $type ('hour', 'day', 'week', 'month') {
        $config{'show_back_'.$type} || next;
        $hrefs.="<a href=\"$device-".$type.$config{'output_ext'}."\"><img border=\"0\" alt=\"Poslednich ".int($config{'show_back_'.$type}).' '.$type.($config{'show_back_'.$type}>1 ? '':'')." - graficka statistika\" src=\"graph-$type.png\"></a>\n";
      }
      $hrefs && ($hrefs="<a href=\"$device-graph".$config{'output_ext'}."\"><img border=\"0\" alt=\"Kompletni graficka statistika rozhrani\" src=\"graph.png\"></a>\n".$hrefs);

      $tabledata.=
        "  <tr valign=\"top\" align=\"right\" ".$config{'output_ttag_throw'}.">\n".
        "    <td><b>Celkem</b></td>\n".
        "    <td>".&NTAFunc::nicenum($data{'text'}{$device}{$date}{'in'})."&nbsp;B<br><b>".&NTAFunc::b2xb($data{'text'}{$device}{$date}{'in'})."B</b></td>\n".
        "    <td>".&NTAFunc::nicenum($data{'text'}{$device}{$date}{'out'})."&nbsp;B<br><b>".&NTAFunc::b2xb($data{'text'}{$device}{$date}{'out'})."B</b></td>\n".
        "  </tr>\n".
        "</table>\n".
        "</dd></dl>\n".
        "<p>\n".
        "$hrefs\n".
        "<p>\n";

      print MONTH 
        $templates{'header'}.
        &NTAFunc::table($templates{'table'}, $netdevices{$device}." ($device)", $tabledata).
        $templates{'footer'};
      close(MONTH);
    }
  }

  ## Graphic output

  my %graphoutputs=();
  foreach my $type ('hour', 'day', 'week', 'month') {
    if($config{'show_back_'.$type}) {
      foreach my $device (keys %netdevices) {
        my $graphdata={};
        @{$graphdata->{'vals_in'}}=();
        @{$graphdata->{'vals_out'}}=();
        %{$graphdata->{'labels'}}=();

        foreach my $date (sort {$b cmp $a} keys %{$data{$type}{$device}}) {
          ($date eq 'last') && next;
          foreach my $direction ('in', 'out') {
            push @{$graphdata->{'vals_'.$direction}}, $data{$type}{$device}{$date}{$direction};
            foreach ($type) {
              /^hour$/ && do {
                $date=~/ ([0-9]+):([0-9]{2})$/ || last;
                ($direction eq 'in') && ($2==0) && ($1%2==0) && 
                  ($graphdata->{'labels'}{$#{$graphdata->{'vals_'.$direction}}}=int($1));
                last;
              };
              /^day$/ && do {
                $date=~/-([0-9]{2}) ([0-9]{2}):([01])$/ || last;
                ($direction eq 'in') && ($2==0) && ($3==0) && do {
                  ($graphdata->{'labels'}{$#{$graphdata->{'vals_'.$direction}}}=int($1).'.');
                };
                last;
              };
              /^week$/ && do {
                $date=~/^([0-9]{4})-([0-9]{2})-([0-9]{2}) ([0-9]{2})$/ || last;
                ($direction eq 'in') && ($4==0) && (strftime("%w", localtime(mktime(0, 0, 0, $3, $2-1, $1-1900)))==0) &&
                  ($graphdata->{'labels'}{$#{$graphdata->{'vals_'.$direction}}}=strftime("%a %b", localtime(mktime(0, 0, 0, $3, $2-1, $1-1900))).' '.int($3).'.');
                last;
              };
              /^month$/ && do {
                $date=~/^([0-9]{4})-([0-9]{2})-([0-9]{2}) ([01])$/ || last;
                ($direction eq 'in') && ($4==0) && ($3==1) && 
                  ($graphdata->{'labels'}{$#{$graphdata->{'vals_'.$direction}}}=strftime("%b 1.", localtime(mktime(0, 0, 0, $3, $2-1, $1-1900))));
                last;
              };
            }
          }
        }

        my $graph=NTAGraph->new(\%config, $graphdata);
        my $coef;
        foreach ($type) {
          /^hour$/ && ($coef=5*60);
          /^day$/ && ($coef=30*60);
          /^week$/ && ($coef=2*3600);
          /^month$/ && ($coef=12*3600);
        }
        $graph->draw($config{'output_dir'}.'/'.$device.'-'.$type.'.png', $coef);

        open(DATA, ">".$config{'output_dir'}.'/'.$device.'-'.$type.$config{'output_ext'}) || &NTAFunc::out("Nemohu zapsat do ".$config{'output_dir'}.'/'.$device.'-'.$type.$config{'output_ext'});
        my $topic="Poslednich ".int($config{'show_back_'.$type}).' '.$type.($config{'show_back_'.$type}>1 ? '':'')." - graficka statistika";
        my $image=
          "<dl>\n".
          "  <dt><b>$topic</b></dt>\n".
          "  <dd><p><a href=\"$device-$type".$config{'output_ext'}."\"><img border=\"0\" alt=\"\" src=\"$device-$type.png\"></a></dd>\n".
          "</dl>\n";

        my $hrefs;
        foreach my $xtype ('hour', 'day', 'week', 'month') {
          $config{'show_back_'.$xtype} || next;
          if ($xtype ne $type) {
            $hrefs.="<a href=\"$device-".$xtype.$config{'output_ext'}."\"><img border=\"0\" alt=\"Poslednich ".int($config{'show_back_'.$xtype}).' '.$xtype.($config{'show_back_'.$xtype}>1 ? '':'')." -  graficka statistika\" src=\"graph-$xtype.png\"></a>\n";
          } else {
            $hrefs.="<img border=\"0\" alt=\"Poslednich ".int($config{'show_back_'.$xtype}).' '.$xtype.($config{'show_back_'.$xtype}>1 ? '':'')." - graficka statistika\" src=\"graph-$xtype.png\">\n";
          }
        }
        $hrefs && ($hrefs="<a href=\"$device-graph".$config{'output_ext'}."\"><img border=\"0\" alt=\"Kompletni graficka statistika rozhrani\" src=\"graph.png\"></a>\n".$hrefs);

        $tabledata=
          "<table border=\"0\">".
          ($newdata{$device}{'ip'} ? "<tr><td>IP adresa: </td><td><b>".$newdata{$device}{'ip'}."</b></td></tr>\n":'').
          ($newdata{$device}{'mac'} ? "<tr><td>MAC adresa: </td><td><b>".$newdata{$device}{'mac'}."</b></td></tr>\n":'').
          "</table>".
          $image.
          "<p>\n".
          "$hrefs\n".
          "<p>\n";

        $graphoutputs{$device}{$type}=$image;
        print DATA
          $templates{'header'}.
          &NTAFunc::table($templates{'table'}, $netdevices{$device}." ($device) - $topic", $tabledata).
          $templates{'footer'};
        close(DATA);

      }

    }
  }

  foreach my $device (keys %netdevices) {
    open(DATA, ">".$config{'output_dir'}."/$device-graph".$config{'output_ext'}) || &NTAFunc::out("Nemohu zapsat do ".$config{'output_dir'}."/$device-graph".$config{'output_ext'});
    $tabledata=
      "<table border=\"0\">".
      ($newdata{$device}{'ip'} ? "<tr><td>IP adresa: </td><td><b>".$newdata{$device}{'ip'}."</b></td></tr>\n":'').
      ($newdata{$device}{'mac'} ? "<tr><td>MAC adresa: </td><td><b>".$newdata{$device}{'mac'}."</b></td></tr>\n":'').
      "</table>";
    foreach my $type ('hour', 'day', 'week', 'month') {
      $graphoutputs{$device}{$type} || next;
      $tabledata.=$graphoutputs{$device}{$type};
    }
    print DATA
      $templates{'header'}.
      &NTAFunc::table($templates{'table'}, $netdevices{$device}." ($device) - Kompletni graficka statistika", $tabledata).
      $templates{'footer'};
    close(DATA);
  }

  foreach my $type ('hour', 'week', 'day', 'month') {
    $config{'show_back_'.$type} || next;
    open(DATA, ">".$config{'output_dir'}."/$type".$config{'output_ext'}) || &NTAFunc::out("Nemohu zapsat do ".$config{'output_dir'}."/$type".$config{'output_ext'});

    $tabledata="<dl>\n";
    foreach my $device (keys %netdevices) {
      $graphoutputs{$device}{$type} || next;
      $tabledata.=
        "<dt>\n".
        "  <b>".$netdevices{$device}." ($device)</b><br>\n".
        "<table border=\"0\">".
        ($newdata{$device}{'ip'} ? "<tr><td>IP adresa: </td><td><b>".$newdata{$device}{'ip'}."</b></td></tr>\n":'').
        ($newdata{$device}{'mac'} ? "<tr><td>MAC adresa: </td><td><b>".$newdata{$device}{'mac'}."</b></td></tr>\n":'').
        "</table>".      
        "</dt>\n".
        "<dd><p><a href=\"$device-$type".$config{'output_ext'}."\"><img border=\"0\" alt=\"\" src=\"$device-$type.png\"></a><p></dd>\n\n";
    }
    $tabledata.="</dl>\n";

    print DATA
      $templates{'header'}.
      &NTAFunc::table($templates{'table'}, "Poslednich ".int($config{'show_back_'.$type}).' '.$type.($config{'show_back_'.$type}>1 ? '':'')." - kompletni graficka statistika", $tabledata).
      $templates{'footer'};
    close(DATA);
  }

  ## Copy images to output directory

  if(opendir(IMGDIR, 'images')) {
    foreach my $file (grep { (-f "images/$_") && /\.(png|gif|jpg|jpeg)$/i } readdir(IMGDIR)) {
      (-f $config{'output_dir'}."/$file") ||
        `$config{'bin_cp'} images/$file $config{'output_dir'}/`;
    }
    closedir(IMGDIR);
  }

  &NTALock::unlock($config{'file_lock'});

  exit 1;
